/*
clientack.h
Copyright (C) 2002  Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2004 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY. 
*/
#ifndef _BHCLIENTACK_H_
#define _BHCLIENTACK_H_

#include "config.h"
#include <pthread.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/time.h>
#include <cstdio>
#include <cerrno>
#include <cstring>

#ifndef _BHGRP_H_
#include "bhgrp.h"
#endif

#include "pos.h"

namespace bhpos{

/**
 * \defgroup bhservice Services
 * \@{
 *
 * \ingroup serverlib
 * \@{	
 */
namespace service{

/**
 * }@
 * \ingroup bhservice
 * \@{
 */
	
class ServiceException
{           
public:
   enum ErrorCode{ 
      seMemory, seBadProtocol, seServiceByName, 
      seBadHostName, seNoIPAddress, seDNSError,
      seMutexLock, seParameter, seOption,
      seNoProtocolSupport, seKernelMemory, seNoDescriptors,
      sePermissionDenied, seInvalidProtocol,
      seBadDescriptor, seBadPointer, seAlreadyConnected,
      seConnectRefused, seConnectTimeOut, seNetUnreachable,
      seAddrInUse, seInProgress, seAlreadyConnecting,
      seIncorrectAddrFamily, seBrdCastNotEnabled, seAlreadyBound,
      seAddressProtected, seCantListen, seNotStreamSock,
      seNoPendingConnections, seFirewall, seNotConnected,
      seWouldBlock, seIntseupted, seMessageSizeTooBig, 
      seNotBound, seOptionNotSupported,
      seWhoCares, seTotalWanker, seUnknown };

   ServiceException(ErrorCode type);
   ServiceException(const ServiceException&);
   ~ServiceException();

   operator const ErrorCode() { return type; }

private:
   ErrorCode  type;
   char       *ErrorString;

   void SetString(const char *pcErrString);
};

class CClientAck{
private:
   int   client;
   bool  bAck;
   
public:
   CClientAck();
   ~CClientAck();
   CClientAck(int, bool);
   CClientAck(const CClientAck& ca);
   CClientAck &operator=(const CClientAck& ca);
   bool operator==(const CClientAck& ca);
   bool operator!=(const CClientAck& ca);
   bool operator==(const int i);
   bool operator!=(const int i);
   bool operator<(const CClientAck& ca);
   bool Status();
   void Ack();
   void Clear();
};

class CClientAckCollector
{
private:
   std::list<CClientAck> clients;
   int current;
   int site;
   int dept;
   int group;
   int post;

public:
   CClientAckCollector();
   CClientAckCollector(int s, int d, int g, int p);
   ~CClientAckCollector();
   void Clear();
   void Reset();
   int Add(CClientAck& ca);
   int Ack(int);
   int Status();
   int GetNext();
   int NumLeft();
   int GetSite() { return site; }
   int GetDepartment() { return dept; }
   int GetGroup() { return group; }
   int GetPost() { return post; }
};
/* }@ */
}
/* }@ 
	}@ */
}

#endif //   _BHCLIENTACK_H_
