/*
clientinfo.h 
Copyright (C) 2001  Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   Don't write to me at: kingsley@bananahead.com 
   Part of:
   BhPos, Copyright (C) 2001-2005 Al Kingsley
   BhPos comes with ABSOLUTELY NO WARRANTY.
*/

#ifndef _CLIENTINFO_
#define _CLIENTINFO_

#ifndef _MSG_STUFF_
#include "bhgrp.h"
#endif

#ifndef _CCASH_H_
#include "cash.h"
#endif

#ifndef _CTRANSITEM_
#include "ctransitems.h"
#endif

/**
 * @defgroup serverlib Server Libraries
 * @{
 */
namespace bhpos{

/**
 * @defgroup clientinfo Client Information
 * @{
 *
 * \ingroup serverlib
 * @{
 */

#define POST_STATUS_NULL            0
#define POST_STATUS_INIT            1
#define POST_STATUS_BOOTING         2
#define POST_STATUS_DISCONNECTED    3
#define POST_STATUS_CONNECTED       4
#define POST_STATUS_LOGOUT          5
#define POST_STATUS_LOGIN           6
#define POST_STATUS_IDLE            7
#define POST_STATUS_TRANSACTION     8
#define POST_STATUS_SHUTDOWNINPROG  9
#define POST_STATUS_SHUTDOWNACK     10
#define POST_STATUS_EMSHUTDOWN      11
#define POST_STATUS_STOREDTRANS     13

#define POST_MODE_NOUSER            0
#define POST_MODE_TRANSNEW          1
#define POST_MODE_SALE              2
#define POST_MODE_PAYMENT           3
#define POST_MODE_TRANSEND          4
#define POST_MODE_LOOKUP            5
#define POST_MODE_GOODSIN           6
#define POST_MODE_GOODSOUT          7
#define POST_MODE_RETURNS           8
#define POST_MODE_LAYBYSUMMARY      9
#define POST_MODE_LAYBYADDPAY       10
#define POST_MODE_EPHEMERALSUMMARY  11
#define POST_MODE_EPHEMERALRECON    12

#define BHUI_PARAM_USERID        1
#define BHUI_PARAM_NUMNOSALE     2
#define BHUI_PARAM_NUMVOID       3
#define BHUI_PARAM_NUMTRANS      4
#define BHUI_PARAM_STARTAMOUNT   5
#define BHUI_PARAM_PAYINAMOUNT   6
#define BHUI_PARAM_PAYOUTAMOUNT  7
#define BHUI_PARAM_TRANSAMOUNT   8
#define BHUI_PARAM_NUMLOGIN      9
#define BHUI_PARAM_STARTTIME     10
#define BHUI_PARAM_CURRENTTIME   11
#define BHUI_PARAM_ENDTIME       12
#define BHUI_PARAM_TOTALTIME     13

#define BHCI_PARAM_STATUS        1
#define BHCI_PARAM_MODE          2
#define BHCI_PARAM_SESSID        3
#define BHCI_PARAM_USERID        4
#define BHCI_PARAM_STATIONID     5
#define BHCI_PARAM_CASHSET       6
#define BHCI_PARAM_CASHADJUST    7
#define BHCI_PARAM_CASHADD       8
#define BHCI_PARAM_CASHSUBTRACT  9
#define BHCI_PARAM_INCTRANS      10
#define BHCI_PARAM_DECTRANS      11
#define BHCI_PARAM_MAC           12
#define BHCI_PARAM_LTIDEMARK     13
#define BHCI_PARAM_HTIDEMARK     14
#define BHCI_PARAM_TRANSOPEN     15
#define BHCI_PARAM_MULTITRANS    16
#define BHCI_PARAM_CASHSTART     17
#define BHCI_PARAM_TRANSSUM      18
#define BHCI_PARAM_LOGIN         19
#define BHCI_PARAM_LOGOUT        20
#define BHCI_PARAM_TRANSVOID     21
#define BHCI_PARAM_NOSALE        22
#define BHCI_PARAM_CUSTID        23
#define BHCI_PARAM_DURATION      24
#define BHCI_PARAM_CURRENCYID    25
#define BHCI_PARAM_LOCALEID      26
#define BHCI_PARAM_SITEID        27
#define BHCI_PARAM_TIDEMARK      28

#define BHCI_NUM_USERS      100
#define BHCI_NUM_ITEMS      101
#define BHCI_NUM_UITEMS     102
#define BHCI_NUM_LAYITEMS   103
#define BHCI_NUM_RETITEMS   104
#define BHCI_NUM_RENTITEMS  105
#define BHCI_NUM_PITEMS     106
#define BHCI_NUM_TAX        107
#define BHCI_NUM_DISCOUNT   108
#define BHCI_NUM_TOTAL      109
#define BHCI_NUM_DELIVERY   110
#define BHCI_NUM_FLAGS      111
#define BHCI_NUM_CASHIND    112

#define BHUI_TRANS_NORMAL   1
#define BHUI_TRANS_NOSALE   2
#define BHUI_TRANS_VOIDSALE 3

/* }@
 * \ingroup clientinfo
 * @{
 */

class CClientInfo;

/*! \brief A class to handle user information.*/
class CUserInfo{
   
   friend class CClientInfo;
   
protected:
   int      ID;
   char     name[32];
   int      num_nosale;
   int      num_voidtrans;
   int      num_trans;
   double   start_amount;
   double   payin_amount;
   double   payout_amount;
   double   total_transacted;
   int      num_login;
	int		num_seconds;
   time_t   start;
   time_t   current;
   time_t   end;
   time_t   total;

public:
   CUserInfo();
	CUserInfo(int);
   CUserInfo(const CUserInfo&);
   ~CUserInfo();
   CUserInfo &operator=(const CUserInfo&);
   bool operator==(CUserInfo const&);
   bool operator!=(CUserInfo const&);
   bool operator==(int const&);
   bool operator!=(int const&);
   int GetParameter(unsigned int, void *);
   int SetParameter(unsigned int, void *);
   void Reset();
   int  Login();
   int  Logout();
   int  AddTrans(double, int);
   void FillData(BhServiceUserSumItem *);
   void FillData(BhBlindBalanceReply *h);
   void ClockIn();
};
/* }@ 
 * \ingroup clientinfo
 * @{
 */
/*! \brief A class to handle client information.*/
class CClientInfo{
private:
   unsigned int status;
   unsigned int mode;
   BhHdr        hdr;

   std::list<CUserInfo> users;
   CStation        station;
   CCash           cash;

   int   Error;
   int   dbID;
   int   userID;
   bool  bTransLeaveOpen;
   bool  bHandleMultiTrans;

   int currencyID;
	int localeID;   
   //int customerID;
   int duration;

   int      num_transactions;
   double   cash_start;
   double   cash_in_drawer;
   double   upper_tidemark, lower_tidemark;
   time_t   last_msg;
   time_t   client_start;
   time_t   user_logon;

   int      sd;
   unsigned char macaddr[6];  // i.e.  00:11:22:33:44:55
   
   time_t   now;

   bool     bKeepalive;
   char     retries;
   char     max_retries;
   
   // request stuff
   time_t   request_rx;
   int      request_type;
   
   double start_amount;
   double payin_amount;
   double payout_amount;
   
   int    num_users;
   int    num_cash;
   double total_cash;
   int    num_cheque;
   double total_cheque;
   int    num_token;
   double total_token;
   int    num_credit;
   double total_credit;
   int    num_smartc;
   double total_smartc;
   int    num_voucher;
   double total_voucher;
   int    num_coupon;
   double total_coupon;
   int    num_other;
   double total_other;
   BhTransCommit ts;
   
   pthread_mutex_t   busy;

protected:
   CTransStack trans;
   
   bool  Lock();
   bool  Release();
   
public:
   CClientInfo();
   CClientInfo(const CClientInfo&);
   virtual ~CClientInfo();
   CClientInfo &operator=(const CClientInfo&);
   bool operator==(CClientInfo const&);
   bool operator!=(CClientInfo const&);
   bool operator<(CClientInfo const&);
   
   void Reset();
   
   int GetParameter(unsigned int, void *);
   int SetParameter(unsigned int, void *);

   void  FlagKeepalive(bool);
   
   void      SetStation(CStation&);
   CStation& GetStation();
   int       AddMonetaryUnit(CMoney&);
   void      SetCash(int unit, float value, int amount);
   void      ClearCash();
   void      FillMcHdr(BhMcPosMsg *);
   int       GetNumber(int, int *, double *);

   int    UpTime();
   int    NumTransactions();
   
   int    NumTrans();
   int    NumItems(int, BhItemType);
   int    NewTrans(int tid, int);
   bool   GetTransaction(int, CTransItems&);
   int    GetNameID(int);
   bool   TransCommit(BhTransCommit *);
   bool   TransPLU(int, CSaleItem&);
   bool   TransPLU(int, BhTransItem *);
   bool   TransPLU(int, BhTransPluCommit *);
   bool   TransPLU(int, BhTransItemUPLU *);
   bool   TransPLU(int, BhRetPLUreply *);
   bool   TransPay(BhTransPaymentCommit *);
   bool   TransPay(int, CPayItem&);
   bool   UpdateTrans(int, CTransItems&);
   bool   IsDelivery();
   void   ClearTrans(int);
   void   ClearAllTrans();
   bool   SetHdr(BhHdr *);
   BhHdr *GetHdr();
   int    UpdateHdr(BhHdr *);
   int    GetLastMsgID();
   void   SetLastMsgID(int);
   
   void   GetEnteredAt(char *);
   void   FillUserInfo(BhPosMsg *);
   
	bool  UserExists(int);
	bool  AddUser(int);
   bool  GetUser(CUserInfo&);
   bool  SetUser(CUserInfo&);
   void  RemoveUser(int);
   void  RemoveUser(CUserInfo&);
	void  UserStartClock(int, bool);
   void  ResetStart();
   void  ResetUserTime();
   bool  SetSystemTime(time_t *);
   bool  SetUserTime(time_t *);
   void  SetRequest(int);
   int   GetRequest(time_t *);
   void  AcknowledgeRequest();
};

/* }@ 
	}@ */

}

/* }@ */

#endif // _CLIENTINFO_
