/*
cevent.cpp
Copyright (C)2001  Al Kingsley


  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY.

*/

#include <sys/time.h>
#include "cevent.h"

namespace bhpos{

CTimeEvent::CTimeEvent()
{
	evtime=0;
	handle=0;
	type=EVENT_TYPE_PERIODIC;
	count=0;
	bEnabled=false;
}

CTimeEvent::CTimeEvent(CTimeEvent const& te)
{
	evtime=te.evtime;
	handle=te.handle;
	type=te.type;
	count=te.count;
	bEnabled=te.bEnabled;
}

CTimeEvent::CTimeEvent(int t, int h, BhEventType y, int c)
{
	evtime=t;
	handle=h;
	type=y;
	count=c;
	bEnabled=false;
}
CTimeEvent::CTimeEvent(timeval *t, int h, BhEventType y, int c)
{
	evtime=(t->tv_sec*1000000) + t->tv_usec;
	handle=h;
	type=y;
	count=c;
	bEnabled=false;
}

CTimeEvent::~CTimeEvent()
{
}

void CTimeEvent::operator=(CTimeEvent const& te)
{
	evtime=te.evtime;
	handle=te.handle;
	type=te.type;
	count=te.count;
	bEnabled=te.bEnabled;
}

bool CTimeEvent::operator==(CTimeEvent const& te)
{
	if(evtime==te.evtime && handle==te.handle && type==te.type) return true;
	return false;
}

bool CTimeEvent::operator!=(CTimeEvent const& te)
{
	if(evtime!=te.evtime || handle!=te.handle || type!=te.type) return true;
	return false;
}

bool CTimeEvent::operator<(const unsigned int t)
{
	if(evtime < t) return true;
	return false;
}
bool CTimeEvent::operator>(const unsigned int t)
{
	if(evtime > t) return true;
	return false;
}
bool CTimeEvent::operator<(timeval const& t)
{
	if(evtime < ((t.tv_sec*1000000) + t.tv_usec)) return true;
	return false;
}
bool CTimeEvent::operator>(timeval const& t)
{
	if(evtime > ((t.tv_sec*1000000) + t.tv_usec)) return true;
	return false;
}
bool CTimeEvent::operator<(CTimeEvent const& t)
{
	if(evtime < t.evtime) return true;
	return false;
}
bool CTimeEvent::operator>(CTimeEvent const& t)
{
	if(evtime > t.evtime) return true;
	return false;
}

void CTimeEvent::Set(int t, BhEventType y, int c)
{
	evtime=t;
	handle=0;
	type=y;
	count=c;
	bEnabled=false;
}

BhEventType CTimeEvent::GetType()
{
	return type;
}
void CTimeEvent::SetType(BhEventType t)
{
	type=t;
}
void CTimeEvent::SetHandle(int h)
{
	handle=h;
}
int CTimeEvent::GetHandle()
{
	return handle;
}
int CTimeEvent::DecCount()
{
	return --count;
}
unsigned int CTimeEvent::GetTime()
{
	return evtime;
}
void CTimeEvent::SetTime(unsigned int t)
{
	evtime=t;
}
bool CTimeEvent::IsEnabled()
{
	return bEnabled;
}
void CTimeEvent::Enable(bool b)
{
	bEnabled=b;
}


///////////////////////////////////////////////////////////////////
//
//
//
//
//

CTEShed::CTEShed()
{
	resolution=10000;
	start=now=max=0;
	
	bRunning=false;
	resettype=0;
	bInterpolate=false;
}

CTEShed::CTEShed(unsigned int res, bool bi, int r)
{
	max=0;
	resolution=res;
	bInterpolate=bi;
	resettype=r;
}

CTEShed::~CTEShed()
{
	events.clear();
}

void CTEShed::SetResolution(unsigned int res, bool bi)
{
	resolution=res;
	bInterpolate=bi;
}

void CTEShed::Start(bool b)
{
	timeval t;

	bRunning=b;

	if(gettimeofday(&t, NULL) < 0){
		bRunning=false;
	}
	else{
		start=(t.tv_sec*1000000) + t.tv_usec;
	}
}

void CTEShed::Reset()
{
	std::list<CTimeEvent>::iterator te;
	timeval t;

	start=0;
	now=0;

	if(gettimeofday(&t, NULL) < 0){
		return;
	}
	else{
		start=((t.tv_sec*1000000) + t.tv_usec);
	}

	for(te=events.begin(); te!=events.end(); te++){
		te->Enable(true);
	}
}

void CTEShed::Tick()
{
	std::list<CTimeEvent>::iterator te;
	CTimeEvent ev;
	timeval	  t;

	if(gettimeofday(&t, NULL) < 0){
		return;
	}
	else{
		now=((t.tv_sec*1000000) + t.tv_usec)-start;
	}
}

int CTEShed::Add(CTimeEvent& ev)
{
	int handle=events.size()+1;
	
	if(!events.empty()){
		if(ev.GetTime() > max){
			max=ev.GetTime();
		}
	}
	ev.SetHandle(handle);
	events.push_back(ev);
	return handle;
}

int CTEShed::Add(CTimeEvent& ev, int h)
{
	int handle=events.size()+1;
	
	if(!events.empty()){
		if(ev.GetTime() > max){
			max=ev.GetTime();
		}
	}
	events.push_back(ev);
	return handle;
}

int CTEShed::Edit(CTimeEvent& oldev, CTimeEvent& newev)
{
	return 0;
}

int CTEShed::Delete(CTimeEvent& ev)
{
	std::list<CTimeEvent>::iterator te=find(events.begin(), events.end(), ev);
	
	if(te==events.end()) return 1;
	
	events.erase(te);
	return 0;
}

int CTEShed::hasEvent(CTimeEvent& collect)
{
	std::list<CTimeEvent>::iterator te;
	int handle=1;

	if(!events.empty()){
		for(te=events.begin(); te!=events.end(); te++){
			if(bInterpolate){
			}
			else{
				if(te->IsEnabled()){
					if(te->GetTime() > now){
						te->Enable(false);	// disable event
						collect=*te;
						if(te->GetType()==EVENT_TYPE_ONESHOT){
							events.erase(te);
						}
						else if(te->GetType()==EVENT_TYPE_NREPEAT){
							if(te->DecCount()==0){
								events.erase(te);
							}
						}
						return handle;
					}
				}
			}
			handle++;
		}
	}
	if(now > max){
		for(te=events.begin(); te!=events.end(); te++){
			if(te->GetType()!=EVENT_TYPE_ONESHOT){
				te->Enable(true);
			}
		}
		Reset();
	}
	return 0;
}

}
