/*
cevent.h
Copyright (C)2001  Al Kingsley


  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY.

*/

#ifndef _BHEVENTER_
#define _BHEVENTER_

#include <list>
#include <algorithm>

namespace bhpos{

/**
 * \defgroup event Event Scheduler
 * \@{
 *
 * \ingroup serverlib
 * \@{
 */

enum BhEventType{
   EVENT_TYPE_ONESHOT,
   EVENT_TYPE_NREPEAT,
   EVENT_TYPE_PERIODIC
};

class CTimeEvent{
private:
	unsigned long evtime;
	int	  handle;
	BhEventType	  type;
	int	  count;
	bool	  bEnabled;
	
public:
	CTimeEvent();
	CTimeEvent(CTimeEvent const&);
	CTimeEvent(int, int, BhEventType, int);
	CTimeEvent(timeval*, int, BhEventType, int);
	~CTimeEvent();
	void operator=(CTimeEvent const&);
	bool operator==(CTimeEvent const&);
	bool operator!=(CTimeEvent const&);
	bool operator<(const unsigned int);
	bool operator>(const unsigned int);
	bool operator<(timeval const&);
	bool operator>(timeval const&);
	bool operator<(CTimeEvent const&);
	bool operator>(CTimeEvent const&);
	
	void Set(int, BhEventType, int);
	BhEventType GetType();
	void SetType(BhEventType);
	void SetHandle(int);
	int GetHandle();
	int DecCount();
	unsigned int GetTime();
	void SetTime(unsigned int);
	bool IsEnabled();
	void Enable(bool);
};

enum BhEventShed{
   SHED_AUTO,
   SHED_MANUAL
};

class CTEShed{
private:
	std::list<CTimeEvent> events;
	unsigned int resolution;
	unsigned int start, now;
	unsigned int max;

	bool bRunning;
	int  resettype;
	bool bInterpolate;
	
public:
	CTEShed();
	CTEShed(unsigned int, bool, int);
	~CTEShed();
	void SetResolution(unsigned int, bool);
	void Start(bool);
	void Reset();
	void Tick();
	int Add(CTimeEvent&);
	int Add(CTimeEvent&, int);
	int Edit(CTimeEvent&, CTimeEvent&);
	int Delete(CTimeEvent&);
	int hasEvent(CTimeEvent&);
};

/* }@ 
   }@ */
}

#endif // _BHEVENTER_
