/*
module.h
Copyright (C)2002 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2005 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY.
*/
#ifndef _MODULE_H_
#define _MODULE_H_

#ifndef _BHQUEUE_T_
#include "pos.h" 
#endif // _BHQUEUE_T_

#include <list>
#include <algorithm>

#include <bhmodule.h>

namespace bhpos{
	
/**
 * @defgroup module Plug-in Modules
 * @{
 *
 * \ingroup serverlib
 * @{
 */
/*! \brief A msg header union.*/  
typedef union{
   unsigned char bmsg[4];
   unsigned int  imsg;
}bhm;
   
/* }@
 * \ingroup module
 * @{
 */

typedef int (*Initialize_t)(void *, void *, void *, void *);
typedef int (*CleanUp_t)();
typedef int (*GetNumMsgHandlers_t)();
typedef int (*GetMsgHandler_t)(int i);
typedef int (*Handler_t)(unsigned int msg_type, int len, void *data, int flags);
typedef bool (*Lock_t)();
typedef bool (*Release_t)();

/*! \brief A class to handle modules.*/
class CModuleHandler{
protected:
	char	      *fname;
	void	      *mod_handle;
   PLUServer   *plus;

   unsigned int  mod_h;
   unsigned int  i_func;
   unsigned int  c_func;
   unsigned int  g_func;
   unsigned int  j_func;
   unsigned int  h_func;

	std::list<unsigned int>		MsgTypes;

	// standard functions
	int   (*Initialize)(void *, void *, void *, void *);
	int   (*CleanUp)();
	int   (*GetNumMsgHandlers)();
	int   (*GetMsgHandler)(int i);
	int   (*Handler)(unsigned int msg_type, int len, void *data, int flags);
	bool  (*Lock)();
	bool  (*Release)();
	
	int   (*QueueExists)(bhpos::BhModQ); 
   int   (*QueueAdd)(bhpos::BhModQ, BhPosMsg *);
	int   (*GetParameter)(int p, void *v);

	int	GetHandlers();

public:
	CModuleHandler();
	CModuleHandler(CModuleHandler const&);
	CModuleHandler(PLUServer *, char *name, void *, void *x, void *y, void *z);
	~CModuleHandler();
	void operator=(CModuleHandler const&);
	bool operator==(CModuleHandler const&);
	bool operator<(CModuleHandler const&);
	
	void Close();
	int  Load(PLUServer *, char *, void *, void *x, void *y, void *z);
	int  Unload();
	bool CanHandle(unsigned int m);
	int  Process(unsigned int msg_type, int len, void *data, int flags);
};

/* }@
 * \ingroup module
 * @{
 */
/*! \brief A class to handle module collections.*/
class CModuleCollection{
protected:
	std::list<CModuleHandler>	Module;
	
public:
	CModuleCollection();
	CModuleCollection(CModuleCollection const&);
	CModuleCollection(PLUServer *, char *dir, void *arg, void *x, void *y, void *z);
	~CModuleCollection();
	void operator=(CModuleCollection const&);
	bool operator==(CModuleCollection const&);
	bool operator<(CModuleCollection const&);
	
	bool ScanDirectory(PLUServer *, char *dir, void *arg, void *x, void *y, void *z);
	void Close();

	int	HandleMsg(unsigned int msg_type, int len, void *data, int flags);
	bool  HasModule(char *modname);
	void *GetModuleFunction(char *modname, char *modfunc);
};

/* }@ 
	}@ */

}

#endif	//	_MODULE_H_
