/*
bhmysql.cpp 
Copyright (C) 2004  Al Kingsley


  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2004 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY. 

*/
#include <cstring>
#include <pthread.h>
#include <cstdio>
#include <cstdarg>
#include <cstdlib>
#include <cstring>
#include "bhmysql.h"

pthread_mutex_t sqlmutex=PTHREAD_MUTEX_INITIALIZER;

namespace bhpos{

BhSQL::BhSQL(char *h, char *dbnamer, unsigned short pt, char *u, char *p)
{
	bConnected=false;
	db=NULL;

	if(h){
		host=(char *) malloc(strlen(h)+1);
		if(host) strcpy(host, h);
	}
	else{
		host=NULL;
	}

	if(dbnamer){
		dbname=(char *) malloc(strlen(dbnamer)+1);
		if(dbname) strcpy(dbname, dbnamer);
	}
	else{
		dbname=NULL;
	}

	if(u){
		user=(char *) malloc(strlen(u)+1);
		if(user) strcpy(user, u);
	}
	else{
		user=NULL;
	}

	if(p){
		password=(char *) malloc(strlen(p)+1);
		if(password) strcpy(password, p);
	}
	else{
		password=NULL;
	}

	user_type=0;
	port=0;
	unix_socket=NULL;
	clientflag=0;
}
BhSQL::~BhSQL()
{
	if(host) free(host);
	if(dbname) free(dbname);
	if(user) free(user);
	if(password) free(password);
	
	pthread_mutex_destroy(&sqlmutex);
	
	if(bConnected==false) return;
	//my_thread_end();
	mysql_close(db);
	bConnected=false;
}

int BhSQL::Init()
{
	db=mysql_init(NULL);
	if(db) return 0;
	return 1;
}

int BhSQL::Connect()
{
	int	res;
	
	pthread_mutex_lock(&sqlmutex);
	
	//my_thread_init();

	if(!bConnected){
		if(!db){
			if((res=Init())){
				pthread_mutex_unlock(&sqlmutex);
				return res;
			}
		}
		if(!mysql_real_connect(db, host, user, password, dbname, port, unix_socket, clientflag)){
			pthread_mutex_unlock(&sqlmutex);
			return 1;
		}
	}
	pthread_mutex_unlock(&sqlmutex);
	bConnected=true;
	return 0;
}

bool BhSQL::IsConnected()
{
   return bConnected;
}

int BhSQL::SelectDb(char *dbname)
{
	if(!db) return 1;
	return mysql_select_db(db, dbname);
}
int BhSQL::SimpleQuery(char *query)
{
   if(query==NULL) return -1;
	return mysql_query(db, query);
}

BhSQLResult *BhSQL::Execute(char *format, ...)
{
	va_list ap;
	char sqlCommand[2048];
	pthread_mutex_lock(&sqlmutex);

	va_start(ap, format);
	vsnprintf(sqlCommand, 2048, format, ap);
	va_end(ap);

	pthread_mutex_unlock(&sqlmutex);
	return new BhSQLResult(this, (char *) sqlCommand, true);
}

void BhSQL::Execute(BhSQLResult *res, char *format, ...)
{
	va_list ap;
	char sqlCommand[2048];

	pthread_mutex_lock(&sqlmutex);
	va_start(ap, format);
	vsnprintf(sqlCommand, 2048, format, ap);
	va_end(ap);

	res->init(this, (char *) sqlCommand, true);
	pthread_mutex_unlock(&sqlmutex);
}

int BhSQL::GetStatus(const char *r)
{
	pthread_mutex_lock(&sqlmutex);
	r=mysql_stat(db);
	pthread_mutex_unlock(&sqlmutex);
	return 0;
}

int BhSQL::GetServerInfo(const char *r)
{
	pthread_mutex_lock(&sqlmutex);
	r=mysql_get_server_info(db);
	pthread_mutex_unlock(&sqlmutex);
	return 0;
}

int BhSQL::GetHostInfo(const char *r)
{
	pthread_mutex_lock(&sqlmutex);
	r=mysql_get_host_info(db);
	pthread_mutex_unlock(&sqlmutex);
	return 0;
}

int BhSQL::GetClientInfo(const char *r)
{
	pthread_mutex_lock(&sqlmutex);
	r=mysql_get_client_info();
	pthread_mutex_unlock(&sqlmutex);
	return 0;
}

unsigned int BhSQL::GetProtocolInfo()
{
	return (mysql_get_proto_info(db));
}

const char *BhSQL::ErrorMsg()
{
	if(!db) return error;
	return mysql_error(db);
}

int BhSQL::SetOption(enum mysql_option option, char *arg)
{
	/*
	switch(option){
	case MYSQL_OPT_CONNECT_TIMEOUT:	// unsigned int * Connect timeout in seconds.
//	case MYSQL_OPT_LOCAL_INFILE:
		
	
	case MYSQL_OPT_COMPRESS:			// Not used Use the compressed client/server protocol.
	case MYSQL_OPT_NAMED_PIPE:			// Not used Use named pipes to connect to a MySQL server on NT.
		return mysql_options(db, option, NULL);
		
	case MYSQL_INIT_COMMAND:			//	char * Command to execute when connecting to MySQL server. Will automatically be re-executed when reconnecting.
	case MYSQL_READ_DEFAULT_FILE:		// char * Read options from the named option instead of from 'my.cnf'.
	case MYSQL_READ_DEFAULT_GROUP:	// char * Read options from the named group from `my.cnf'.or the specied with MYSQL_
		return mysql_options(db, option, (char *) arg);
	}
	*/
	return mysql_options(db, option, arg);
}


/////////////////////////////////////////////////////////////////////////
//
//	Generic db result
//
//

BhSQLResult::BhSQLResult()
{
	result=NULL;
	Error=0;
	bStored=false;
}

BhSQLResult::BhSQLResult(BhSQL *db, char *query, bool bStore)
{
	result=NULL;
	curr_row=NULL;
	Error=0;
	init(db, query, bStore);
}

BhSQLResult::~BhSQLResult()
{
	if(result!=NULL)			// Free memory resources
		mysql_free_result(result);
	result=NULL;
}

void BhSQLResult::free()
{
   if(result!=NULL)
		mysql_free_result(result);
		
	result=NULL;
}

int BhSQLResult::init(BhSQL *db, char *query, bool bStore)
{
	Error=0;
	int err;
	if(result!=NULL){
		mysql_free_result(result);
		result=NULL;
	}
	if(bStore){
		bStored=true;
		if((err=mysql_query(db->db, query))==0){
		   //mysql_debug("d:t:O,/tmp/plusd_login.trace");
			result=mysql_store_result(db->db);
			if(result!=NULL){
				if(mysql_num_rows(result)==0){
					return 0;
			   }
			} 
			else{ 
				Error=DB_ERROR;
         }
		}
		else{
			Error=err;
	   }
	}
	else{
		if((err=mysql_query(db->db, query))==0){
			result=mysql_use_result(db->db);
			/*
			if(result!=NULL){
				Error = false;
			} 
			else 
				Error=false;
*/
		}
		else 
			Error=err;
	}
	return Error;
}

int BhSQLResult::status()
{
	if(Error) return Error;
	return DB_ROWS_OK;
}

int BhSQLResult::NumRows()
{
	if(result==NULL) return 0;
	return mysql_num_rows(result);
}

int BhSQLResult::NumFields()
{
	if(result==NULL) return 0;
	return mysql_num_fields(result);
}

char *BhSQLResult::fieldName(int n)
{
	MYSQL_FIELD *field;

	if(result==NULL) return NULL;
	mysql_field_seek(result, n);
	field=mysql_fetch_field(result);
	if(field==NULL) return NULL;
	return field->name;
}

int BhSQLResult::fieldSize(int n)
{
	MYSQL_FIELD *field;

	if (result == NULL) return 0;
	mysql_field_seek(result, n);
	field = mysql_fetch_field(result);
	if (field == NULL) return 0;
	return field->length;
}

int BhSQLResult::fieldSize(char *name)
{
	int i;

	if(result==NULL) return 0;
	for(i=0; i<NumFields(); i++)
		if(!strcmp(name, fieldName(i)))
			return fieldSize(i);
	return 0;
}

void BhSQLResult::SeekRow(int row)
{
	if(result==NULL) return;
	mysql_data_seek(result, row);
}

BhSQLRow *BhSQLResult::GetRow()
{
	MYSQL_ROW row;

	if(result==NULL) return NULL;
	row=mysql_fetch_row(result);
	if(row){
		curr_row=new BhSQLRow(result, row);
		return curr_row;
	}
	else{
		curr_row=NULL;
	}
	return NULL;
}

BhSQLRow *BhSQLResult::GetRow(int row)
{
	SeekRow(row);
	return GetRow();
}

int BhSQLResult::GetLastID(BhSQL *db)
{
	return mysql_insert_id(db->db);
}

//////////////////////////////////////////////////////////////////


BhSQLRow::BhSQLRow(MYSQL_RES *r, MYSQL_ROW f)
{
	result=r;
	row=f;
}

BhSQLRow::~BhSQLRow()
{
	if(!IsClosed()){
		Close();
	}
}
	
char *BhSQLRow::GetField(int f)
{
	if(IsClosed()) return NULL;
	
	if(f < 0 || f >GetNumFields()) return NULL;
	
	return row[f];
}

int BhSQLRow::GetNumFields()
{
	if(IsClosed()) return -1;
	
	return mysql_num_fields(result);	
}

int BhSQLRow::IsClosed()
{
	if(row==NULL) return true;
	
	return false;
}

void  BhSQLRow::Close()
{
	if(IsClosed()) return;
	row=NULL;
	result=NULL;
}

}
