/*
bhmysql.h - wraps mysql
Copyright (C)2004 Al Kingsley

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Don't write to me at: kingsley@bananahead.com 

	Part of:
	BhPos, Copyright (C) 2001-2005 Al Kingsley
	BhPos comes with ABSOLUTELY NO WARRANTY.
*/
#ifndef _BH_MYSQL_H_
#define _BH_MYSQL_H_

#include <pthread.h>
#include <mysql.h>

namespace bhpos{
	
/**
 * @defgroup bhmysql MySql Wrapper
 * @{
 *
 * \ingroup serverlib
 * @{
 */

#define DB_COMMAND_OK		0	// OK - command executed
#define DB_EMPTY_QUERY	   1	// Query didn't return owt
#define DB_ROWS_OK			2	// Query returned rows
#define DB_ERROR			   5
#define DB_BAD_RESPONSE	   6
#define DB_UNEXPECTED		7	// This shouldn't ever happen

#define BH_NOERROR			 0
#define BH_ERROR_FIELDLEN	1

#define BH_USER_TYPE_ADMIN  0
#define BH_USER_TYPE_USER	1
#define BH_USER_TYPE_READER 2

/* }@
 * \ingroup bhmysql
 * @{
 */

class BhSQLResult;
/*! \brief A class to handle mysql.*/
class BhSQL{
protected:
	bool			  bConnected;
	int				user_type;
	char			 *host;
	char			 *dbname;
	char			 *user;
	char			 *password;
	unsigned int	port;
	char			 *unix_socket;
	unsigned int	clientflag;
	char			  error[100];

public:
	MYSQL			*db;
	
	BhSQL(char *h, char *db, unsigned short port, char *u, char *p);
	~BhSQL();
	int  Init();
	int  SetOption(enum mysql_option option, char *arg);
	int  Connect();
	bool IsConnected();
	int  SelectDb(char *);
	int  SimpleQuery(char *);
	BhSQLResult *Execute(char *query, ...);	
	void Execute(BhSQLResult *res, char *query, ...);
	
	int GetStatus(const char *);
	int GetServerInfo(const char *);
	int GetHostInfo(const char *);
	int GetClientInfo(const char *);
	unsigned int GetProtocolInfo();
	const char *ErrorMsg();

};

class BhSQLRow;

/* }@
 * \ingroup bhmysql
 * @{
 */
//! \class BhSQLResult
//! \brief A class to handle mysql results.
class BhSQLResult{
private:
	int			 Error;
	bool			bStored;
	MYSQL_RES	*result;
	BhSQLRow	 *curr_row;

public:
	BhSQLResult();
	BhSQLResult(BhSQL *db, char *query, bool bStore);
	~BhSQLResult();
	void free();

	int init(BhSQL *db, char *query, bool bStore);
	
	int status();						// Return query status
		
	int NumRows();						// Number of fetched rows
	int NumFields();					// Number of fields per rows

	char *fieldName(int n);			// Name of nth field
	int   fieldSize(int n);			// Size of nth field
	int   fieldSize(char *name);	// Size of nth field

	
	void SeekRow(int row);			// Sets internal cursor to row
	BhSQLRow *GetRow();
	BhSQLRow *GetRow(int row);

	int GetLastID(BhSQL *db);
};

/* }@
 * \ingroup bhmysql
 * @{
 */
/*! \brief A class to handle mysql rows.*/
class BhSQLRow{
private:
	MYSQL_RES	*result;
	MYSQL_ROW	row;

public:
	BhSQLRow();
	BhSQLRow(MYSQL_RES *, MYSQL_ROW);
	~BhSQLRow();
	
	char *GetField(int);
	int	GetNumFields();
	int	IsClosed();
	void  Close();
};

/* }@
	}@ */
}


#endif //_BH_MYSQL_H_
